-- using modified code from  https://blogs.msdn.microsoft.com/sqlserverstorageengine/2016/01/10/clustered-columnstore-index-data-load-optimizations-minimal-logging/
-- log measuring code from http://sqlmag.com/t-sql/minimally-logged-inserts

USE PLSSUG_95_SIMPLE;
GO

IF OBJECT_ID(N'dbo.t_source', N'U') IS NOT NULL
	DROP TABLE dbo.t_source;
GO

CREATE TABLE dbo.t_source (
	accountkey				int not null,
	accountdescription		char (500),
	accounttype				char(500),
	AccountCodeAlternatekey	int
)
CHECKPOINT
GO

BEGIN TRANSACTION

--DECLARE
--	@numrecords AS INT,
--	@SIZE AS BIGINT,
--	@dt AS DATETIME
--;
                              
--SELECT
--	@numrecords = COUNT(*),
--	@SIZE       = COALESCE(SUM([Log Record Length]), 0),
--	@dt         = CURRENT_TIMESTAMP
--FROM sys.fn_dblog(NULL, NULL) d
--WHERE
--	AllocUnitName = 'dbo.t_source' OR AllocUnitName LIKE 'dbo.t_source.%'


DECLARE @i INT = 0

WHILE (@i < 110000)
BEGIN
	INSERT INTO dbo.t_source VALUES (@i, 'description', 'dummy-accounttype', @i * 2)
	SET @i = @i + 1
END

--SELECT
--	numrecords		= COUNT(*) - @numrecords,
--	size_mb			= CAST((COALESCE(SUM([Log Record Length]), 0) - @SIZE) / 1024. / 1024. AS NUMERIC(12, 2)),
--	duration_sec	= CAST(DATEDIFF(millisecond, @dt, CURRENT_TIMESTAMP)/1000. AS DECIMAL(12,3))
--FROM sys.fn_dblog(NULL, NULL) d
--WHERE
--	AllocUnitName = 'dbo.t_source' OR AllocUnitName LIKE 'dbo.t_source.%'

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_source' OR AllocUnitName LIKE 'dbo.t_source.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context
;
COMMIT TRANSACTION

-- ===================== INSERT SELECT ==========================

IF OBJECT_ID(N'dbo.t_destination', N'U') IS NOT NULL
	DROP TABLE dbo.t_destination;
GO

CREATE TABLE dbo.t_destination (
	accountkey				int not null,
	accountdescription		char (500),
	accounttype				char(500),
	AccountCodeAlternatekey	int
)
CHECKPOINT
GO

BEGIN TRANSACTION

--DECLARE
--	@numrecords AS INT,
--	@SIZE AS BIGINT,
--	@dt AS DATETIME
--;
                              
--SELECT
--	@numrecords = COUNT(*),
--	@SIZE       = COALESCE(SUM([Log Record Length]), 0),
--	@dt         = CURRENT_TIMESTAMP
--FROM sys.fn_dblog(NULL, NULL) d
--WHERE
--	AllocUnitName = 'dbo.t_destination' OR AllocUnitName LIKE 'dbo.t_destination.%'

INSERT INTO dbo.t_destination
	(accountkey, accountdescription, accounttype, AccountCodeAlternatekey)
SELECT
	accountkey, accountdescription, accounttype, AccountCodeAlternatekey
FROM dbo.t_source

--SELECT
--	numrecords		= COUNT(*) - @numrecords,
--	size_mb			= CAST((COALESCE(SUM([Log Record Length]), 0) - @SIZE) / 1024. / 1024. AS NUMERIC(12, 2)),
--	duration_sec	= CAST(DATEDIFF(millisecond, @dt, CURRENT_TIMESTAMP)/1000. AS DECIMAL(12,3))
--FROM sys.fn_dblog(NULL, NULL) d
--WHERE
--	AllocUnitName = 'dbo.t_destination' OR AllocUnitName LIKE 'dbo.t_destination.%'

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_destination' OR AllocUnitName LIKE 'dbo.t_destination.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context
;

COMMIT TRANSACTION

-- ================= INSERT SELECT + TABLOCK =======================

IF OBJECT_ID(N'dbo.t_destination', N'U') IS NOT NULL
	DROP TABLE dbo.t_destination;
GO

CREATE TABLE dbo.t_destination (
	accountkey				int not null,
	accountdescription		char (500),
	accounttype				char(500),
	AccountCodeAlternatekey	int
)

CHECKPOINT
GO

BEGIN TRANSACTION

--DECLARE
--	@numrecords AS INT,
--	@SIZE AS BIGINT,
--	@dt AS DATETIME
--;
                              
--SELECT
--	@numrecords = COUNT(*),
--	@SIZE       = COALESCE(SUM([Log Record Length]), 0),
--	@dt         = CURRENT_TIMESTAMP
--FROM sys.fn_dblog(NULL, NULL) d
--WHERE
--	AllocUnitName = 'dbo.t_destination' OR AllocUnitName LIKE 'dbo.t_destination.%';

INSERT INTO dbo.t_destination  WITH (TABLOCK)
	(accountkey, accountdescription, accounttype, AccountCodeAlternatekey)
SELECT
	accountkey, accountdescription, accounttype, AccountCodeAlternatekey
FROM dbo.t_source

--SELECT
--	numrecords		= COUNT(*) - @numrecords,
--	size_mb			= CAST((COALESCE(SUM([Log Record Length]), 0) - @SIZE) / 1024. / 1024. AS NUMERIC(12, 2)),
--	duration_sec	= CAST(DATEDIFF(millisecond, @dt, CURRENT_TIMESTAMP)/1000. AS DECIMAL(12,3))
--FROM sys.fn_dblog(NULL, NULL) d
--WHERE
--	AllocUnitName = 'dbo.t_destination' OR AllocUnitName LIKE 'dbo.t_destination.%';

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_destination' OR AllocUnitName LIKE 'dbo.t_destination.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context
;

COMMIT TRANSACTION
