-- using modified code from  https://blogs.msdn.microsoft.com/sqlserverstorageengine/2016/01/10/clustered-columnstore-index-data-load-optimizations-minimal-logging/
-- log measuring code from http://sqlmag.com/t-sql/minimally-logged-inserts

-- ===== SQL 2016 =====

USE PLSSUG_95_FULL;
GO

IF OBJECT_ID(N'dbo.t_source', N'U') IS NOT NULL
	DROP TABLE dbo.t_source;
GO

CREATE TABLE dbo.t_source (
	accountkey				int not null,
	accountdescription		char (500),
	accounttype				char(500),
	AccountCodeAlternatekey	int
)

BEGIN TRANSACTION
DECLARE @i int = 0

WHILE (@i < 110000)
	BEGIN
		INSERT INTO dbo.t_source VALUES (@i, 'description', 'dummy-accounttype', @i * 2)
		SET @i = @i + 1
	END
COMMIT TRANSACTION

-- clean up the transaction log
CHECKPOINT
BACKUP LOG PLSSUG_95_FULL
TO DISK = 'C:\Program Files\Microsoft SQL Server\MSSQL13.SQL2016\MSSQL\Backup\PLSSUG_95_FULL.trn'
WITH INIT
GO

SELECT COUNT(*) FROM sys.fn_dblog(NULL, NULL)


-- ===== SELECT INTO =====
IF OBJECT_ID(N'dbo.t_destination', N'U') IS NOT NULL
	DROP TABLE dbo.t_destination;
GO

BEGIN TRANSACTION

SELECT *
INTO dbo.t_destination
FROM dbo.t_source

--DBCC IND ('PLSSUG_95_FULL', 'dbo.t_destination', 1)
--DBCC TRACEON (3604)
--DBCC PAGE ('PLSSUG_95_FULL', 1, 28029, 3)
--DBCC TRACEOFF (3604)

--SELECT *
--FROM sys.fn_dblog(NULL, NULL) t_log
--WHERE
--	allocunitname = 'dbo.t_destination' 
--	--AND operation LIKE '%LOP_INSERT_ROWS%'
--	-- take a look at: LOP_FORMAT_PAGE + LCK_HEAP

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_destination' OR AllocUnitName LIKE 'dbo.t_destination.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	AvgLen,
	Operation,
	Context
;

COMMIT TRANSACTION

DROP TABLE dbo.t_destination;
GO

-- ============================= SIMPLE =================
USE PLSSUG_95_SIMPLE;
GO


IF OBJECT_ID(N'dbo.t_source', N'U') IS NOT NULL
	DROP TABLE dbo.t_source;
GO

CREATE TABLE dbo.t_source (
	accountkey				int not null,
	accountdescription		char (500),
	accounttype				char(500),
	AccountCodeAlternatekey	int
)

BEGIN TRANSACTION
DECLARE @i int = 0

WHILE (@i < 110000)
	BEGIN
		INSERT INTO dbo.t_source VALUES (@i, 'description', 'dummy-accounttype', @i * 2)
		SET @i = @i + 1
	END
COMMIT TRANSACTION

-- clean up the transaction log
CHECKPOINT
SELECT COUNT(*) FROM sys.fn_dblog(NULL, NULL)

-- ===== SELECT INTO ===== 
IF OBJECT_ID(N'dbo.t_destination', N'U') IS NOT NULL
	DROP TABLE dbo.t_destination;
GO

BEGIN TRANSACTION

SELECT *
INTO dbo.t_destination
FROM dbo.t_source


--DBCC IND ('PLSSUG_95_SIMPLE', 't_destination', 1)
--DBCC TRACEON (3604)
--DBCC PAGE ('PLSSUG_95_SIMPLE', 1, 6712, 3)
--DBCC TRACEOFF (3604)

--SELECT *
--FROM sys.fn_dblog(NULL, NULL) t_log
--WHERE
--	allocunitname = 'dbo.t_destination' 
--	--AND operation LIKE '%LOP_INSERT_ROWS%'
--	-- take a look at: LOP_FORMAT_PAGE + LCK_HEAP

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_destination' OR AllocUnitName LIKE 'dbo.t_destination.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	AvgLen,
	Operation,
	Context
;

COMMIT TRANSACTION

DROP TABLE dbo.t_destination;
