-- https://blogs.msdn.microsoft.com/sqlserverstorageengine/2008/10/23/new-update-on-minimal-logging-for-sql-server-2008/
-- log measuring code from http://www.sqlservercentral.com/articles/Stairway+Series/94552/



USE PLSSUG_95_SIMPLE;
GO

-- create the source table, this table stores 2 row per page
IF OBJECT_ID(N'dbo.t_source', N'U') IS NOT NULL
	DROP TABLE dbo.t_source;
GO

CREATE TABLE dbo.t_source (
	c1	INT,
	c2	INT,
	c3	CHAR (3000)
)
GO

-- insert 10000 rows; with 2 rows/page, table will have 5000 pages so
-- the size of the table is 40MB

BEGIN TRANSACTION
DECLARE
	@i	INT	= 1
;

WHILE (@i < 10000)
BEGIN
	INSERT INTO dbo.t_source VALUES (@i, @i + 10000, 'hello');
	SELECT @i = @i + 1
END
COMMIT TRANSACTION
 

-- create target table with clustered index
IF OBJECT_ID(N'dbo.t_ci', N'U') IS NOT NULL
	DROP TABLE dbo.t_ci;
GO
CREATE TABLE t_ci (
	c1	INT,
	c2	INT,
	c3	CHAR(3000)
)
GO

CREATE CLUSTERED INDEX ci ON t_ci(c1)
GO


-- now insert all the rows from the source table into the target table
-- and measure (a) logspace used (b)logspace reserved (c) number of writes
-- both with TF-610 ON and OFF.

CHECKPOINT

BEGIN TRANSACTION

INSERT INTO dbo.t_ci
SELECT * 
FROM dbo.t_source
ORDER BY c1

SELECT
	d.name,
	d.recovery_model_desc,
	LogRecordCount		= database_transaction_log_record_count,
	LogMBUsed			= ROUND(database_transaction_log_bytes_used / 1024. / 1024., 2),
	LogMBReserved		= ROUND(database_transaction_log_bytes_reserved / 1024. / 1024., 2),
	SecondsToRebuild	= DATEDIFF(ss, database_transaction_begin_time, GETDATE())
FROM    sys.dm_tran_database_transactions AS dt
        INNER JOIN sys.dm_tran_session_transactions AS st
               ON dt.transaction_id = st.transaction_id
        INNER JOIN sys.databases AS d ON dt.database_id = d.database_id
WHERE d.name = 'PLSSUG_95_SIMPLE'

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_ci' OR AllocUnitName LIKE 'dbo.t_ci.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context

ROLLBACK TRANSACTION

-- ================ TF 610 ===============
USE PLSSUG_95_SIMPLE;
GO

-- create the source table, this table stores 2 row per page
IF OBJECT_ID(N'dbo.t_source', N'U') IS NOT NULL
	DROP TABLE dbo.t_source;
GO

CREATE TABLE dbo.t_source (
	c1	INT,
	c2	INT,
	c3	CHAR (3000)
)
GO

-- insert 10000 rows; with 2 rows/page, table will have 5000 pages so
-- the size of the table is 40MB

BEGIN TRANSACTION
DECLARE
	@i	INT	= 1
;

WHILE (@i < 10000)
BEGIN
	INSERT INTO dbo.t_source VALUES (@i, @i + 10000, 'hello');
	SELECT @i = @i + 1
END
COMMIT TRANSACTION
 

-- create target table with clustered index
IF OBJECT_ID(N'dbo.t_ci', N'U') IS NOT NULL
	DROP TABLE dbo.t_ci;
GO
CREATE TABLE t_ci (
	c1	INT,
	c2	INT,
	c3	CHAR(3000)
)
GO

CREATE CLUSTERED INDEX ci ON t_ci(c1)
GO

-- now insert all the rows from the source table into the target table
-- and measure (a) logspace used (b)logspace reserved (c) number of writes
-- both with TF-610 ON and OFF.

CHECKPOINT

DBCC TRACEON (610, -1)

BEGIN TRANSACTION

INSERT INTO dbo.t_ci
SELECT * 
FROM dbo.t_source
ORDER BY c1

SELECT
	d.name,
	d.recovery_model_desc,
	LogRecordCount		= database_transaction_log_record_count,
	LogMBUsed			= ROUND(database_transaction_log_bytes_used / 1024. / 1024., 2),
	LogMBReserved		= ROUND(database_transaction_log_bytes_reserved / 1024. / 1024., 2),
	SecondsToRebuild	= DATEDIFF(ss, database_transaction_begin_time, GETDATE())
FROM    sys.dm_tran_database_transactions AS dt
        INNER JOIN sys.dm_tran_session_transactions AS st
               ON dt.transaction_id = st.transaction_id
        INNER JOIN sys.databases AS d ON dt.database_id = d.database_id
WHERE d.name = 'PLSSUG_95_SIMPLE'

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_ci' OR AllocUnitName LIKE 'dbo.t_ci.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context

ROLLBACK TRANSACTION

DBCC TRACEOFF (610, -1)