-- using modified scripts from http://www.sqlservercentral.com/articles/Stairway+Series/94552/

USE PLSSUG_95_FULL;
GO

IF OBJECT_ID('dbo.PrimaryTable_Large', 'U') IS NOT NULL 
    DROP TABLE dbo.PrimaryTable_Large
GO

CREATE TABLE dbo.PrimaryTable_Large (
	ID			INT			IDENTITY PRIMARY KEY ,
	SomeColumn	CHAR(4)		NULL ,
	Filler		CHAR(1500)	DEFAULT ''
);
GO

INSERT INTO dbo.PrimaryTable_Large (
	SomeColumn
)
SELECT TOP 100000
	'abcd '
FROM msdb.sys.columns a

	CROSS JOIN msdb.sys.columns b
GO

SELECT  *
FROM sys.dm_db_index_physical_stats(
	DB_ID(N'PLSSUG_95_FULL'),
    OBJECT_ID(N'PrimaryTable_Large'),
    NULL,
	NULL,
	'DETAILED'
);

--truncate the log
USE master
GO
BACKUP LOG PLSSUG_95_FULL
TO DISK = 'C:\Program Files\Microsoft SQL Server\MSSQL13.SQL2016\MSSQL\Backup\PLSSUG_95_FULL.trn'
WITH INIT
GO

-- rebuild index and interrogate log space use, within a transaction
USE PLSSUG_95_FULL
GO

BEGIN TRANSACTION 

ALTER INDEX ALL ON dbo.PrimaryTable_Large REBUILD
 -- there's only the clustered index

SELECT
	DatabaseName		= d.name,
	RecoveryModel		= d.recovery_model_desc,
	LogRecords			= database_transaction_log_record_count,
	MBused				= ROUND(database_transaction_log_bytes_used / 1024. / 1024., 2),
	SecondsToRebuild	= DATEDIFF(ss, database_transaction_begin_time, GETDATE())
FROM sys.dm_tran_database_transactions dt

	INNER JOIN sys.dm_tran_session_transactions st
		ON dt.transaction_id = st.transaction_id

	INNER JOIN sys.databases d
		ON dt.database_id = d.database_id

WHERE
	d.name = 'PLSSUG_95_FULL'

COMMIT TRANSACTION

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.PrimaryTable_Large' OR AllocUnitName LIKE 'dbo.PrimaryTable_Large.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context
;

-- ====== SIMPLE =====

USE PLSSUG_95_SIMPLE;
GO

IF OBJECT_ID('dbo.PrimaryTable_Large', 'U') IS NOT NULL 
    DROP TABLE dbo.PrimaryTable_Large
GO

CREATE TABLE PrimaryTable_Large (
	ID INT IDENTITY
			PRIMARY KEY ,
	SomeColumn CHAR(4) NULL ,
	Filler CHAR(1500) DEFAULT ''
);
GO

INSERT INTO dbo.PrimaryTable_Large (
	SomeColumn
)
SELECT TOP 100000
	'abcd '
FROM msdb.sys.columns a

	CROSS JOIN msdb.sys.columns b
GO

SELECT  *
FROM sys.dm_db_index_physical_stats(
	DB_ID(N'PLSSUG_95_SIMPLE'),
    OBJECT_ID(N'PrimaryTable_Large'),
    NULL,
	NULL,
	'DETAILED'
);

-- ====
-- rebuild index and interrogate log space use, within a transaction

CHECKPOINT

BEGIN TRANSACTION 

ALTER INDEX ALL ON dbo.PrimaryTable_Large REBUILD
 -- there's only the clustered index

SELECT
	DatabaseName		= d.name,
	RecoveryModel		= d.recovery_model_desc,
	LogRecords			= database_transaction_log_record_count,
	MBused				= ROUND(database_transaction_log_bytes_used / 1024. / 1024., 2),
	SecondsToRebuild	= DATEDIFF(ss, database_transaction_begin_time, GETDATE())
FROM sys.dm_tran_database_transactions dt

	INNER JOIN sys.dm_tran_session_transactions st
		ON dt.transaction_id = st.transaction_id

	INNER JOIN sys.databases d
		ON dt.database_id = d.database_id

WHERE
	d.name = 'PLSSUG_95_SIMPLE'
COMMIT TRANSACTION

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.PrimaryTable_Large' OR AllocUnitName LIKE 'dbo.PrimaryTable_Large.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context
;