-- using modified code from  https://blogs.msdn.microsoft.com/sqlserverstorageengine/2016/01/10/clustered-columnstore-index-data-load-optimizations-minimal-logging/
-- ========== SQL 2014 ==========

USE PLSSUG_95_SIMPLE;
GO

IF OBJECT_ID(N'dbo.t_bulkload', N'U') IS NOT NULL
	DROP TABLE dbo.t_bulkload;
GO

-- create the table
CREATE TABLE dbo.t_bulkload (
	accountkey				int not null,
	accountdescription		char (500),
	accounttype				char(500),
	AccountCodeAlternatekey	int
)

-- Let us prepare the data
-- insert 110K rows into a regular table

BEGIN TRAN

DECLARE @i int = 0

WHILE (@i < 110000)
	BEGIN
		INSERT INTO dbo.t_bulkload VALUES (@i, 'description', 'dummy-accounttype', @i * 2)
		SET @i = @i + 1
	END
COMMIT

-- bcp out the data run the following in command window
-- C:\Users\Administrator>bcp PLSSUG_95_SIMPLE..t_bulkload out c:\temp\t_bulkoad-2014.dat -c -T -S .\SQL2014

-- truncate the table
TRUNCATE TABLE dbo.t_bulkload

-- create clustered columnstore index
CREATE CLUSTERED COLUMNSTORE index t_bulkload_cci on dbo.t_bulkload

-- clean up the transaction log (note, the database is in simple recovery mode)
CHECKPOINT
GO

--SELECT COUNT(*) FROM sys.fn_dblog(NULL, NULL)

-- now bulkload the data
BEGIN TRAN

-- this loads the data in 1 batch
BULK INSERT dbo.t_bulkload FROM 'c:\temp\t_bulkoad-2014.dat'

-- show the log records generated
-- note, I am using 'log record length 1104' because I know the length of the
-- insert log record.

--SELECT *
----COUNT(*)
--FROM sys.fn_dblog(NULL, NULL) t_log
--WHERE
--	allocunitname LIKE '%t_bulkload%' 
--	AND operation LIKE '%LOP_INSERT_ROWS%'
--	AND [Log Record Length] = 1104 -- dla SQL 2014 jest inny rozmiar rekordu

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_bulkload' OR AllocUnitName LIKE 'dbo.t_bulkload.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context
;

ROLLBACK TRAN

CHECKPOINT
GO

-- You will see that it returns 0 as no data rows were logged. 
-- Abort the transaction and run the checkpoint command to cleanup the log records. 
-- Now let us run the same command but with the batchsize of 10000. 
-- Recall, this will load the rows into delta rowgroup

BEGIN TRAN
-- Now use a smaller batchsize of 10000
-- I am using even TABLOCK

BULK INSERT dbo.t_bulkload
   FROM 'c:\temp\t_bulkoad-2014.dat'
   WITH (
    BATCHSIZE = 10000,
	TABLOCK
)

-- show the log records generated
--SELECT COUNT(*)
--	--*
--FROM sys.fn_dblog(NULL, NULL) t_log
--WHERE
--	allocunitname LIKE '%t_bulkload%' 
--	AND operation LIKE '%LOP_INSERT_ROWS%'
--	AND Context = 'LCX_CLUSTERED'		-- 110K
--	--AND [Log Record Length] = 1104 -- dla SQL2014 daje inne rozmiary rekordw

SELECT
	Operation,
	Context,
	AvgLen		= AVG([Log Record Length]),
	Cnt			= COUNT(*)
FROM sys.fn_dblog(NULL, NULL) d
WHERE
	AllocUnitName = 'dbo.t_bulkload' OR AllocUnitName LIKE 'dbo.t_bulkload.%'
GROUP BY
	Operation,
	Context,
	ROUND([Log Record Length], -2)
ORDER BY
	Operation,
	Context
;

ROLLBACK
